import datetime
import requests
import os
import json
import urllib.parse


class LoggerApi:
    def __init__(self, url, estado):

        mode = os.environ['MODE']

        self.url = url
        self.estado = estado
        self.APIURL = os.getenv("APIURL")

        if mode != 'TEST':
            self.routes = self.obtener_indices_pasos()

    def informar(self, msg):
        data = {'nombre': self.estado,
                'fecha': datetime.datetime.now().strftime("%d/%m/%Y %H:%M:%S"),
                'observaciones': msg}

        requests.post(self.url,
                      headers={"Content-Type": "application/json",
                               "Authorization": self.obtener_credenciales_pipelines()},
                      json=data)

    def cancel_job(self, urld):
        urld = urld.rstrip('pasosEjecucion/')

        # build url endpoint with the ID of the step and the process
        data = {'estado': 'Erroneo'}

        mode = os.environ['MODE']

        if mode not in ['TEST']:
            # POST data
            r = requests.delete(urld, headers={"Authorization": self.obtener_credenciales_pipelines()}, data=data)
            print(r)

    def return_ignore_steps(self):
        ignore_steps = """TagVariants
Gender_check
Report_client
report_client
FindAlu
AnnotatePGx
MappingRef
OffTargetCNVs
MappingUmi
MsiAnalysis
SvFusionsReMap
Expression_Counts
MapAlu
CleanBamAlu
Homopolymer
VariantCallingVsRef
VariantAnnotationVsRef
CreateXlsxVsRef
PGxGenerateRegionsFile
DnaExpression
PGxVariantCalling
AnnotatePGx
CoveragePerBaseRef
NormalizeWithRef
PlotCancerCnvs
SortSam
MolecularBarcoding
SvFusionsExtract
SvFusionsCall
FusionsFilter
FusionsTransform""".split('\n')

        return ignore_steps

    def iniciar_paso(self, name, analysis, log):

        ignore_steps = self.return_ignore_steps()

        if name in ignore_steps:
            return
        # build data
        data = {'fecha': datetime.datetime.now().strftime("%d/%m/%Y %H:%M:%S")}

        # build url endpoint with the ID of the step and the process
        try:
            url = urllib.parse.urljoin(self.url, self.routes[analysis][name] + '/inicio')
        except:
            log.warn(f"Paso iniciar - {analysis} - {name} no implementado en la base de datos")
            return

        # POST data
        r = requests.post(url, headers={"Content-Type": "application/json",
                                        "Authorization": self.obtener_credenciales_pipelines()},
                          json=data)

        # log.debug(f"POST [{name}]: data -> {data}")
        log.debug(f"#[START]: API-POST [{name}]: status -> {r.status_code}; url -> {url}")

    def finalizar_paso(self, name, analysis, log):

        ignore_steps = self.return_ignore_steps()

        if name in ignore_steps:
            return
        # build data
        data = {'fecha': datetime.datetime.now().strftime("%d/%m/%Y %H:%M:%S")}

        # build url endpoint with the ID of the step and the process
        try:
            url = urllib.parse.urljoin(self.url, self.routes[analysis][name] + '/finalizacion')
        except:
            log.warn(f"Paso finalizar - {analysis} - {name} no implementado en la base de datos")
            return

        # POST data
        r = requests.post(url,
                          headers={"Content-Type": "application/json",
                                   "Authorization": self.obtener_credenciales_pipelines()},
                          json=data)

        # log.debug(f"POST [{name}]: data -> {data}")
        log.debug(f"#[END]: API-POST [{name}]: status -> {r.status_code}; url -> {url}")

    def obtener_indices_pasos(self):
        # Select the endpoint direction from which get the data

        api_url = f'{self.APIURL}/pipelineapi/v1/pipelines'

        # Process the get request to the API
        r = requests.get(api_url, headers={
                                           "Content-Type": "application/json",
                                           "Authorization": self.obtener_credenciales_pipelines()
                                           })

        data = json.loads(r.text)

        # Filter out inactive pipelines
        active_pipelines = []

        for pipeline in data:
            if pipeline['estado'] != 'activa':
                continue

            # Add the active pipeline to the list
            active_pipelines.append(pipeline)

        routes = {}

        # For each of the pipelines, get the data of each step, and create a
        # dictionary with it
        for pipeline in active_pipelines:
            pipeline_id = str(pipeline['id'])

            r = requests.get(api_url + '/' + pipeline_id,
                             headers={
                                      "Content-Type": "application/json",
                                      "Authorization": self.obtener_credenciales_pipelines()
                                      })

            data = json.loads(r.text)

            routes[pipeline['nombre']] = {step['nombre']: str(step['id']) for step in data['pasos']}

        return routes

    def mandar_path_resultados(self, config):
        # Build data
        data = config['results']

        # build url endpoint
        url = self.url.rstrip('pasosEjecucion/') + '/resultados'

        # POST data
        r = requests.post(url, headers={"Content-Type": "application/json",
                                        "Authorization": self.obtener_credenciales_pipelines()}, json=data)

    def obtener_credenciales_pipelines(self):

        api_url = f'{self.APIURL}/seguridadapi/v1/login'

        data = {
                "usuario": "pipelineuser",
                "password": "pipelinepassword",
                "dominio": "pipelineapi"
                }

        r = requests.post(api_url, headers={"Content-Type": "application/json"}, json = data)
        auth = r.content.decode('utf-8')
        return "Bearer " + auth

    def empty_cache(self, mode):
        # print('#[LOG]: emptying cache')
        if mode != 'TEST':
            try:
                os.popen("sh -c 'echo 3 > /proc/sys/vm/drop_caches'", 'w')
            except:
                print("#[WARN]: Could not clean the cache (do python have root privileges?)")



import logging
import coloredlogs
import graypy
import os


class Log:

    def __init__(self):
        self.mode = os.environ['MODE']

    @staticmethod
    def set_mode(args):
        if args.d is True:
            mode = 'Debug'
        else:
            mode = 'Production'

        return mode

    def get_log(self, config: dict, log_file=""):

        # Initial variables
        message = False
        failed_file = False

        # If there is a config dictionary, check the log files
        if bool(config):
            log = self.set_log(type(self).__name__, config['log_files'])
            log.debug(f"Logging into {config['log_files'][0]}")
            return log

        # If not, write to simple file or to stream on stdout
        else:
            # Check if log_file has a value
            if bool(log_file):

                message = ""

                # Check if instance is string
                if not isinstance(log_file, str):
                    message = f"<log_file> variable should be a string instance, not {type(log_file)}"
                    failed_file = True

                # If the folder does not exists, just check the value to true
                if not failed_file and not os.path.exists(os.path.dirname(log_file)):
                    message = f"Output folder to write the log file does not exists ({os.path.dirname(log_file)})"
                    failed_file = True

                # If everything has been ok, write to a file
                if not failed_file:
                    log = self.set_log(type(self).__name__, [log_file])
                    log.info(f"Logging into {log_file}")
                    return log

            # Create a logging stream if no file is found
            log = self.set_log(type(self).__name__)
            log.debug(f"No logging file found, logging into stream")

            if failed_file is True:
                log.warn(message)

        return log

    @staticmethod
    def set_log(name, dirs: list = list()):

        log = logging.getLogger(name)
        log.setLevel(logging.DEBUG)

        if len(dirs) > 0:
            for d in dirs:
                # create a file handler
                handler = logging.FileHandler(d)
                handler.setLevel(logging.DEBUG)
                formatter = logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s')
                handler.setFormatter(formatter)

                log.addHandler(handler)
                level = "INFO"

        else:
            # create a file handler
            handler = logging.StreamHandler()
            handler.setLevel(logging.DEBUG)
            formatter = logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s')
            handler.setFormatter(formatter)

            log.addHandler(handler)
            level = "DEBUG"

        # Create stream handler
        coloredlogs.install(level=level, logger=log)

        log_ip = os.getenv('LOGIP')
        if log_ip is not None:
            handler_gp = graypy.GELFHandler(log_ip, 12201)
            log.addHandler(handler_gp)

        return log



import os
import subprocess
import time
from bimgn.utils.logs import Log
from bimgn.utils.softwares import Software
from bimgn.workflow.it_api import LoggerApi


class BioTask:

    def __init__(self,
                 config: dict,
                 tool_name: str = "",
                 log_file: str = "",
                 debug: bool = False,
                 test: bool = False,
                 dry_run: bool = False):
        """
        :param log_file: path to a writing file
        :param config: dictionary with information of the process
        :param tool_name: name of the tool that is being used
        """

        self.debug = debug
        self.test = test
        self.dry_run = dry_run
        self.config = config

        # Launch the methods that are going to build the entire class configuration
        self.task_config = self.get_basic_config(config)

        self.log = self.get_log(config, log_file)
        self.environment = self.get_mode()
        self.mode = self.get_mode()

        self.software = Software(log=self.log)

        self.pid = 0
        self.max_ram_memory = self.set_memory()
        self.logger = self.set_api_logger(config)

        # Check if configuration has data
        if bool(config):

            self.tool_name = tool_name

            if tool_name is not None:
                self.tool_config = self.config['tools_conf'][tool_name]
                if self.tool_config != {}:
                    pass

            self.log.info(f"Created {type(self).__name__} class")
            self.log.debug(f"{type(self).__name__} - {self.tool_config}")

            self.threads = self.set_threads()

            self.tmp_folder = self.set_tmp_folder()

            self.cmd, self.rm_cmd = "", ""

        self.create_input_output()
        if self.dry_run: self.run_dry_mode()

    def create_input_output(self):
        pass

    @staticmethod
    def get_log(config, log_file):
        """Get the log class for the biotask

        :param config: dictionary with information of the process
        :param log_file: path to a writing file
        :return: log class with writing information
        """

        # Initialize log class
        log_class = Log()
        log = log_class.get_log(config, log_file)
        return log

    def run_dry_mode(self):

        for key in self.tool_config['output'].keys():
            filepath = str(self.tool_config['output'][key])

            if '.' not in filepath:
                self.log.info(f"Creating dry directory -> {filepath}")
                os.system(f"mkdir -p {filepath}")
            else:
                self.log.info(f"Creating dry file -> {filepath}")
                fh = open(filepath, 'w')
                fh.close()

    def get_basic_config(self, config) -> dict:
        """
        Create an initial configuration that will change in some parameters

        :param config: Initial configuration of the tool, if given
        :return: The method will return the dictionary
        """
        options = {
            "execution": {
                "type": "",
                "sample": {
                    "sample_id": "",
                },
                "assay": {
                    "assay_id": ""
                },
                "pipeline_info": {
                    "url": "",
                    "subtype": "",
                    "panel": "",
                },
                "io": {
                    "inputs": {},
                    "outputs": {},
                    "tmp_folder": self.set_tmp_folder()
                },
                "coverage": {
                    "minimum_depth": 0,
                    "minimum_covered": 0,
                    "minimum_offtarget": 0,
                },
                "variant_calling": {
                    "variant_allele_fraction": 0,
                    "minimum_variant_depth": 0
                },
                "molecular_barcoding": {
                    "included": False
                },
                "qc": {
                    "stid": "0000"
                },
                "tool_info": {
                    "threads": self.set_threads(),
                    "max_memory": self.set_memory(),
                    "pid": 0
                },
            },
            "debug": self.debug,
            "testing": self.test,
            "dry_run": self.dry_run
        }

        return options

    def get_mode(self):
        mode = os.environ['MODE']

        if mode in ['', None]:
            self.log.error("#[ERR] -> The environment variable $MODE should be set [TEST, DEV, PREP, PROD]")
            self.log.error("#[ERR] -> E.g. '$ export MODE='TEST'")
            exit(1)

        self.log.debug(f"Environment of work:{mode}")
        return mode

    def set_api_logger(self, config):

        if self.mode != 'TEST':
            logger = LoggerApi(config['url'], type(self).__name__)
        else:
            logger = None

        return logger

    def modify_software_version(self, new_softwares: list):

        # Modification of the software version
        self.software.modify_softwares(new_softwares)

    def set_sample_id(self):
        """Method to return the id of the analyzed case"""

        if "sampleID" in self.config['process_conf']['sample'].keys():
            sample_id = self.config['process_conf']['sample']['sampleID']
        else:
            sample_id = self.config['process_conf']['sample']['trioID']

        return sample_id

    def set_tmp_folder(self):
        """Method to set a temporal folder"""
        if 'tmp_folder' in self.config['process_conf']:
            tmp_folder = self.config['process_conf']['tmp_folder']
        else:
            tmp_folder = "/tmp/"

        return tmp_folder

    def set_threads(self):
        """Method to set the threads used by the tool. Number of threads should be"""
        threads = os.getenv('THREADS')

        if not bool(threads):
            threads = "1"

        try:
            int(threads)
        except ValueError:
            raise Exception("Number of threads could not be converted to integer")

        try:
            assert (int(threads) <= 0)
        except AssertionError as ass_err:
            print(ass_err)
        else:
            pass
        finally:
            pass

        return threads

    def set_memory(self):
        memory = os.getenv("MAX_MEMORY")
        # todo validate memory
        if memory is None:
            # return "-Xmx5g"
            return ""
        else:
            return "-Xmx" + memory

    def set_pid(self, pid):
        self.pid = pid

    def run(self):
        if self.dry_run: return None

        import time
        init_time = time.time()

        if self.config['process_conf']['sample']['modality'] == 'Trios':
            if 'sample' in self.tool_config['tool_conf'].keys():
                name = type(self).__name__ + ' - ' + self.tool_config['tool_conf']['sample']
            else:
                name = type(self).__name__
        else:
            name = type(self).__name__

        # Only talk to the API when a logger exists
        if self.logger is not None:
            self.logger.iniciar_paso(name, self.config['process_conf']['sample']['modality'], self.log)

        # Execute the tool
        self.run_process()

        # Finalyze process and calculate time
        end_time = time.time()
        self.execution_time = str(round(end_time - init_time, 2))
        self.log.debug(f'__time__ - {name} - {self.execution_time} s')

        # Only talk to the API when a logger exists
        if self.logger is not None:
            self.logger.finalizar_paso(name, self.config['process_conf']['sample']['modality'], self.log)
            self.logger.informar(f"{name} result")

    @staticmethod
    def build_rm_cmd():
        return ''

    def build_cmd(self):
        raise Exception("Method 'build_cmd' is  not set.")

    def get_task_options(self):
        return self.tool_config

    def cmd_run(self, mode=3):

        if self.dry_run: return None

        cmd = self.build_cmd()
        rm_cmd = self.build_rm_cmd()
        self.log.debug(cmd)

        # If dry_run, don't run the process, just print it
        if mode == 1:
            self.log.info('Running command...')
            process = subprocess.Popen(cmd, shell=True, executable='/bin/bash',
                                       stdout=subprocess.PIPE,
                                       stderr=subprocess.PIPE)

            self.log.info(f"Initializing process {type(self).__name__} - PID: {process.pid}")
            self.set_pid(process.pid)

            out = None
            err = None
            lines = []

            self.log.debug("Printing software log:")
            while out != "" or err != "":
                out = process.stdout.readline()
                err = process.stderr.readline()
                out = out.decode("utf-8").strip('\n')
                err = err.decode("utf-8").strip('\n')
                self.log.debug(err)
                lines.append(out)

            return lines

        elif mode == 2:
            os.system(cmd)

        elif mode == 3:
            f_stdout = open("/tmp/full.stdout.log", "w+")
            f_stderr = open("/tmp/full.stderr.log", "w+")
            # Using pipe in command could block the stdout, see this post:
            # https://thraxil.org/users/anders/posts/2008/03/13/Subprocess-Hanging-PIPE-is-your-enemy/
            # https://www.reddit.com/r/Python/comments/1vbie0/subprocesspipe_will_hang_indefinitely_if_stdout/
            self.log.info('Running command...')
            process = subprocess.Popen(cmd, shell=True, executable='/bin/bash',
                                       stdout=f_stdout, stderr=f_stderr)
            self.log.info(f"Initializing process {type(self).__name__} - PID: {process.pid}")
            self.set_pid(process.pid)

            while process.poll() is None:
                time.sleep(5)

            f_stdout.close()
            f_stderr.close()

            if process.returncode != 0:

                # If a temporal folder has been used, try to retrieve a removing command
                if rm_cmd != '':
                    self.log.debug(f"Process {type(self).__name__} failed. Command failed running")
                    self.log.debug(f"Activation of removal of temporal files...")
                    f_stdout = open("/tmp/full.stdout.log", "w+")
                    f_stderr = open("/tmp/full.stderr.log", "w+")
                    p = subprocess.Popen(rm_cmd, shell=True, executable='/bin/bash', stdout=f_stdout,
                                         stderr=f_stderr)
                    while process.poll() is None:
                        time.sleep(5)

                    f_stdout.close()
                    f_stderr.close()

                raise Exception(f"Process {type(self).__name__} failed. Command failed running")

        self.log.info(f"Finished process {type(self).__name__} with exit status 0")

    def run_cmd_get_output(self, cmd):
        process = subprocess.Popen(cmd, shell=True, executable='/bin/bash',
                                   stdout=subprocess.PIPE, stderr=subprocess.PIPE)

        out = None
        err = None
        lines = []

        while out != "" or err != "":
            out = process.stdout.readline()
            err = process.stderr.readline()
            out = out.decode("utf-8").strip('\n')
            err = err.decode("utf-8").strip('\n')
            lines.append(out)

        return lines



import subprocess

def run_cmd_get_output(cmd):
    process = subprocess.Popen(cmd, shell=True, executable='/bin/bash',
                               stdout=subprocess.PIPE, stderr=subprocess.PIPE)

    out = None
    err = None
    lines = []

    while out != "" or err != "":
        out = process.stdout.readline()
        err = process.stderr.readline()
        out = out.decode("utf-8").strip('\n')
        err = err.decode("utf-8").strip('\n')
        lines.append(out)

    return lines



import os
import glob


class Configuration:
    def __init__(self):
        self.version = version

        job = postdata['job']
        indir = postdata['input_dir']
        sampleid = postdata['sampleid']
        outfolder = os.path.join(postdata['output_dir'], sampleid + '_IMGJ' + str(postdata['job']))
        self.stid = postdata['stid']
        url = postdata['url']
        panel = postdata['panel']

        self.r1_files = ",".join(glob.glob(os.path.join(indir, str(sampleid) + '*_R1_*')))
        self.r2_files = ",".join(glob.glob(os.path.join(indir, str(sampleid) + '*_R2_*')))

        if self.r1_files == "" or self.r2_files == "":
            raise Exception(f"FASTQ files couldn't been found in the provided directory - " +
                            f"{os.path.join(indir, str(sampleid) + '*_R1_*')}")

        self.r1_trimmed = ",".join([os.path.join(outfolder, 'raw', fastq.split('/')[-1]).replace(
            'fastq.gz', 'trimmed.fastq.gz') for fastq in self.r1_files.split(',')])
        self.r2_trimmed = ",".join([os.path.join(outfolder, 'raw', fastq.split('/')[-1]).replace(
            'fastq.gz', 'trimmed.fastq.gz') for fastq in self.r2_files.split(',')])

        self.maxthreads = os.environ['THREADS']
        self.options = self.set_options(postdata)

        if self.maxthreads is None:
            self.maxthreads = "2"

        self.DEBUG = True
        self.TESTING = False
        self.DRY_RUN = False
        self.sampleID = str(sampleid)
        self.outfolder = outfolder
        self.url = url
        self.ALU = self.options['options_process_alu']
        self.log_files = [os.path.join(outfolder, "logs", str(sampleid) + '.log')]
        self.NM_path = "/DATA/biodata/exome/nm_list.txt"
        self.process_conf = {
            "jobID": str(job),
            "build": "GRCh37",
            "threads": self.maxthreads,
            "tmp_folder": f"/tmp/{job}",
            "main_pipeline": "Exome",
            "sample": {
                "modality": 'Exoma',
                "sampleID": str(sampleid),
                "panel": panel
            }
        }

        self.tools_conf = {}


import os
from bimgn.utils.logs import Log


class Software:
    def __init__(self,
                 log_file: str = "",
                 config: dict = dict(),
                 log=""):

        self.software_data = self.build_standard_config()
        self.log = self.get_log(log=log)

    def get_log(self, log="", config=dict(), log_file=""):
        """Get the log class for the biotask

        :param log: This would be the log class given by other tool
        :param config: dictionary with information of the process
        :param log_file: path to a writing file
        :return: log class with writing information
        """

        if bool(log):
            log.info("Already existing log")
            return log

        # Initialize log class
        log_class = Log()
        log = log_class.get_log(config, log_file)
        return log

    def call(self, software: str):
        """
        This method will return the software path of a specific called software (bwa, vardict...)

        :param software: String that contains the name of the software to call
        :return: path to the software
        """

        # Get the variables of the software
        version = self.software_data[software]['version']
        software_path = self.software_data[software]['path']

        # Write to log
        self.log.info(f"#[INFO][SOFTWARE] - {software},{version},{software_path}")

        # Return the path to the software used
        return self.software_data[software]['path']

    def get_init_var(self):
        """
        Docstring
        """

        ngs_software_bin = os.getenv('BIN_BIOTOOLS')
        refgenomes = os.getenv('REFGENOMES')
        output_dir = os.getenv('OUTPUT_DIR')
        plasmid_ref = os.getenv('PLASMID_REF')
        vep_cache = os.getenv('VEPCACHE')
        biodata = os.getenv('BIODATA')
        mode = os.getenv('MODE')

        init_conf = {
            'BIN_BIOTOOLS': ngs_software_bin,
            'REFGENOMES': refgenomes,
            'OUTPUT_DIR': output_dir,
            'PLASMID_REF': plasmid_ref,
            'VEPCACHE': vep_cache,
            'BIODATA': biodata,
            'MODE': mode
        }

        if any(a is None for a in [ngs_software_bin, refgenomes, output_dir, plasmid_ref]):
            for key in init_conf:
                print(key + ' -> "' + str(init_conf[key]) + '"')
            raise Exception('#[ERR]: One of the variables is not set')

        return init_conf

    def build_standard_config(self, software_name: str = "", software_version: str = ""):

        software_data = self.return_software()
        software_default = self.default_versions()

        softwares = {}

        for soft in software_default:
            soft = software_default[soft]

            for name in software_data[soft['name']][soft['version']].keys():
                softwares[name] = {
                    'path': software_data[soft['name']][soft['version']][name],
                    'version': soft['version']
                }

        for key in software_data['additional_data'].keys():
            softwares[key] = software_data['additional_data'][key]

        return softwares

    def modify_softwares(self, new_softwares: list):

        full_softwares = self.return_software()

        for new_software_version in new_softwares:

            software_main = new_software_version['main_name']
            version = new_software_version['version']
            name = new_software_version['name']

            if software_main not in full_softwares.keys():
                self.log.warn("Software not available")
                return None

            software = full_softwares[software_main]

            if version not in software.keys():
                self.log.warn(f"Version {version} for {name} not available")
                return None

            old_software_version = self.software_data[name]['version']
            new_software_version = version

            if old_software_version != new_software_version:
                self.log.warn(f"Using different version for software {name} " +
                              f"({old_software_version} -> {new_software_version})")
            else:
                self.log.info(f"Using same version for software {name} " +
                              f"({old_software_version} -> {new_software_version})")

            self.software_data[name] = {
                "path": software[version][name],
                "version": version
            }

    def call_change_software_version(self, software_dict):

        tool_softwares = self.assign_softwares()

        for key_software in software_dict.keys():
            software = software_dict[key_software]

            if 'version' in software:
                old_software_version = tool_softwares[software['name']]['version']
                new_software_version = software['version']

                if old_software_version != new_software_version:
                    self.log.warn(f"Using different version for software {software['name']} " +
                                  f"({old_software_version} -> {new_software_version})")
                else:
                    self.log.info(f"Using same version for software {software['name']} " +
                                  f"({old_software_version} -> {new_software_version})")

                try:
                    software_builtin = Software()
                    software_data = software_builtin.return_software()
                    self.tool_software[key_software] = {'path': software_data[software['main_name']][new_software_version][key_software], 'name': key_software, 'main_name': software['main_name'], 'version': new_software_version}

                    return software_data[software['main_name']][new_software_version][software['name']]

                except KeyError:
                    self.log.error(f"#[ERR]: There is no version for the indicated software ({software['name']})")
                    raise Exception(f"#[ERR]: There is no version for the indicated software ({software['name']})")

        return tool_softwares

    def return_software(self):
        init_conf = self.get_init_var()
        ngs_software_bin = init_conf['BIN_BIOTOOLS']
        plasmid_ref = init_conf['PLASMID_REF']
        vep_cache = init_conf['VEPCACHE']

        config = dict()

        config['ref'] = {
            "GRCh38": os.path.join(init_conf['REFGENOMES'], 'hs38', 'hs38.fa'),
            "GRCh37": os.path.join(init_conf['REFGENOMES'], 'hs37d5', 'hs37d5.fa')
        }

        software = {
            "bwa-mem": {
                "v0.7.15": {
                    "bwa-mem": os.path.join(ngs_software_bin, 'bwa')
                }
            },

            "samtools": {
                "v1.5": {
                    "samtools": os.path.join(ngs_software_bin, 'samtools-1.5'),
                },
                "v1.7": {
                    "samtools": os.path.join(ngs_software_bin, 'samtools-1.7'),
                },
                "v1.9": {
                    "samtools": os.path.join(ngs_software_bin, 'samtools-1.9'),
                }
            },

            "picard": {
                "v2.18.15": {
                    "picard": os.path.join(ngs_software_bin, 'picard-2.18.15.jar')
                },
                "v2.9.0": {
                    "picard": os.path.join(ngs_software_bin, 'picard-2.9.0.jar')
                }
            },

            "fgbio": {
                "v0.6.1": {
                    "fgbio": os.path.join(ngs_software_bin, 'fgbio-0.6.1.jar')
                }
            },

            "htslib": {
                "v1.5": {
                    "bgzip": os.path.join(ngs_software_bin, 'bgzip-1.5'),
                    "tabix": os.path.join(ngs_software_bin, 'tabix-1.5'),
                },
                "v1.7": {
                    "bgzip": os.path.join(ngs_software_bin, 'bgzip-1.7'),
                    "tabix": os.path.join(ngs_software_bin, 'tabix-1.7'),
                },
                "v1.9": {
                    "bgzip": os.path.join(ngs_software_bin, 'bgzip-1.9'),
                    "tabix": os.path.join(ngs_software_bin, 'tabix-1.9'),
                }
            },

            "freebayes": {
                "v1.1.0": {
                    "freebayes": os.path.join(ngs_software_bin, 'freebayes'),
                }
            },

            "annovar": {
                "v20160205": {
                    "ANNOVAR": os.path.join(ngs_software_bin, 'table_annovar.pl'),
                }
            },

            "bedtools": {
                "v2.26.0": {
                    "bedtools": os.path.join(ngs_software_bin, 'bedtools'),
                }
            },
            "snpsift": {
                "v4.3k": {
                    "snpsift": os.path.join(ngs_software_bin, 'SnpSift.jar'),
                    "snpeff": os.path.join(ngs_software_bin, 'snpEff.jar'),
                }
            },

            "fastQC": {
                "v0.11.5": {
                    "fastQC": os.path.join(ngs_software_bin, 'fastqc'),
                }
            },

            "bbduk": {
                "v37.56": {
                    "bbduk": os.path.join(ngs_software_bin, 'bbduk.sh'),
                }
            },

            "RTG": {
                "v3.8.4": {
                    "RTG": os.path.join(ngs_software_bin, 'RTG.jar')
                }
            },

            "vcfallelicprim": {
                "v0.0": {
                    "vcfallelicprim": os.path.join(ngs_software_bin, 'vcfallelicprimitives'),
                }
            },

            "vt": {
                "v0.1": {
                    "vt": os.path.join(ngs_software_bin, 'vt'),
                }
            },
            "sort_bed": {
                "v0.1": {
                    "sort_bed": os.path.join(ngs_software_bin, 'sort_bed'),
                }
            },

            "gemini": {
                "v0.20.1": {
                    "gemini": os.path.join(ngs_software_bin, 'gemini'),
                }
            },

            "vep": {
                "v92.1": {
                    "vep": os.path.join(ngs_software_bin, 'vep'),
                }
            },

            "vardict": {
                "v1.5.1": {
                    "vardict": os.path.join(ngs_software_bin, 'VarDict_1.5.1')
                },
                "v1.5.2": {
                    "vardict": os.path.join(ngs_software_bin, 'VarDict_1.5.2')
                },
                "v1.5.3": {
                    "vardict": os.path.join(ngs_software_bin, 'VarDict_1.5.3')
                },
                "v1.5.7": {
                    "vardict": os.path.join(ngs_software_bin, 'VarDict_1.5.7')
                },

            },
            "vardict-script": {
                "v1.5.1": {
                    "vardictsomatic": os.path.join(ngs_software_bin, 'testsomatic.R'),
                    "vardictpaired": os.path.join(ngs_software_bin, 'var2vcf_paired.pl'),
                    "vardictsb": os.path.join(ngs_software_bin, 'teststrandbias.R'),
                    "vardictvar2vcf": os.path.join(ngs_software_bin, 'var2vcf_valid.pl'),
                }
            },

            "igvtools": {
                "v2.3.98": {
                    "igvtools": os.path.join(ngs_software_bin, 'igvtools.jar'),
                }
            },

            "bamclipper": {
                "v1.0.0": {
                    "bamclipper": os.path.join(ngs_software_bin, 'bamclipper.sh'),
                }
            },

            "cutprimers": {
                "v1.2": {
                    "cutprimers": os.path.join(ngs_software_bin, 'cutPrimers.py'),
                }
            },
            "bcl2fastq": {
                "v0.0.0": {
                    "bcl2fastq": os.path.join(ngs_software_bin, 'bcl2fastq'),
                }
            },

            "agent": {
                "v4.0.1": {
                    "agent": os.path.join(ngs_software_bin, 'LocatIt_v4.0.1.jar'),
                }
            },
            "lumpy": {
                "v0.2.12": {
                    "lumpy": os.path.join(ngs_software_bin, 'lumpy'),
                    "lumpy_paired": os.path.join(ngs_software_bin, 'pairend_distro.py'),
                    "lumpy_splitreads": os.path.join(ngs_software_bin, 'extractSplitReads_BwaMem'),
                },
            },
            "samblaster": {
                "v0.1.24": {
                    "samblaster": os.path.join(ngs_software_bin, 'samblaster'),
                }
            },
            "imegen": {
                "v0.0": {
                    "imegen": ""
                }
            },
            "qualimap": {
                "v2.2.1": {
                    "qualimap": os.path.join(ngs_software_bin, 'qualimap')
                }
            },
            "trim_galore": {
                "v0.5.0": {
                    "trim_galore": os.path.join(ngs_software_bin, "trim_galore_0.5.0")
                }
            },

            "additional_data": {
                "v0.0": {
                    "REFERENCE_GENOME": "/DATA/biodata/refGenomes/hs37d5/hs37d5.fa",
                    "HUMANDB": os.path.join(ngs_software_bin, 'annov_humandb'),
                    "NEXTERAPE": os.path.join("/DATA/biodata/NexteraPE-PE.fa"),
                    "VEPCACHE": vep_cache,
                    "VEPDB": os.path.join('/DATA/biodata/vep/'),
                    "PLASMID_REF": os.path.join(plasmid_ref, 'plasm_stidk_seq.fa'),
                    "PLASMID_IDX": os.path.join(plasmid_ref, 'indexes.tsv'),
                    "PLASMID_BED": os.path.join(plasmid_ref, 'regions_stidk.bed'),
                    "IMEGENDB": os.path.join(ngs_software_bin, 'test.db'),
                    "BUILD": "GRCh37"
                }
            }
        }

        return software

    @staticmethod
    def check_config_paths(config):
        """
        Docstring
        """
        for software in config['software']['paths']:
            path = config['software']['paths'][software]
            if isinstance(path, str) and not os.path.lexists(path):
                print("Software " + software + " does not exists -> " + path)
                print("Exiting...")
                exit(1)

        for reference in config['ref']:
            path = config['ref'][reference]
            if isinstance(path, str) and not os.path.exists(path):
                print("#[WARN]: Reference " + reference + " does not exists -> " + path)

        for db in config['dbs']:
            path = config['dbs'][db]
            if isinstance(path, str) and not os.path.exists(path) and db != 'IMEGENDB':
                print("Database " + db + " does not exists -> " + path)
                print("Exiting...")
                exit(1)

        for data in config['software']['data']:
            path = config['software']['data'][data]
            if isinstance(path, str) and not os.path.exists(path):
                print("Data " + data + " does not exists -> " + path)
                print("Exiting...")
                exit(1)

    @staticmethod
    def default_versions():
        using_software = {
            'fastQC': {"name": "fastQC", "version": "v0.11.5"},
            'bbduk': {"name": "bbduk", "version": "v37.56"},
            'qualimap': {"name": "qualimap", "version": "v2.2.1"},
            'trim_galore': {"name": "trim_galore", "version": "v0.5.0"},
            'bwa-mem': {"name": "bwa-mem", "version": "v0.7.15"},
            'fgbio': {'name': 'fgbio', 'version': 'v0.6.1'},
            'samtools': {"name": "samtools", "version": "v1.7"},
            'htslib': {"name": "htslib", "version": "v1.5"},
            'cutprimers': {"name": "cutprimers", "version": "v1.2"},
            'bedtools': {"name": "bedtools", "version": "v2.26.0"},
            'picard': {"name": "picard", "version": "v2.9.0"},
            'igvtools': {"name": "igvtools", "version": "v2.3.98"},
            'vardict': {"name": "vardict", "version": "v1.5.1"},
            'vardict-script': {"name": "vardict-script", "version": "v1.5.1"},
            'RTG': {"name": "RTG", "version": "v3.8.4"},
            'sort_bed': {"name": "sort_bed", "version": "v0.1"},
            'snpsift': {"name": "snpsift", "version": "v4.3k"},
            'vep': {"name": "vep", "version": "v92.1"},
            'lumpy': {"name": "lumpy", "version": "v0.2.12"},
            'imegen': {"name": "imegen", "version": "v0.0"},
            'samblaster': {'name': "samblaster", "version": "v0.1.24"},
            'agent': {'name': 'agent', 'version': 'v4.0.1'},
            'freebayes': {'name': 'freebayes', 'version': 'v1.1.0'},
            'vt': {"name": "vt", "version": "v0.1"},
            'additional_data': {'name': 'additional_data', 'version': 'v0.0'}
        }

        return using_software



import bimgn
import unittest


class Test_001:
    def __init__(self):
        self.task = bimgn.BioTask()

    def test_simple_int(self):
        assert True is 1

    def test_configuration(self):
        assert self.task.max_ram_memory == '5g'


if __name__ == '__main__':
    unittest.main()
